'use strict';

Object.defineProperty(exports, "__esModule", {
    value: true
});
exports.makeIERequest = makeIERequest;
exports.makeStandardRequest = makeStandardRequest;
exports.default = makeRequest;
function makeIERequest(url, _ref) {
    var callback = _ref.callback,
        error = _ref.error,
        method = _ref.method,
        bodyParams = _ref.bodyParams;

    var defaults = {
        callback: function callback() {
            return null;
        },
        error: function error() {
            return null;
        },
        method: 'GET',
        bodyParams: null
    };

    var options = Object.assign({}, defaults, {
        callback: callback,
        error: error,
        method: method,
        bodyParams: bodyParams
    });

    var xdr = new XDomainRequest();

    console.log('making request with XDR');

    xdr.open(options.method.toLowerCase(), url);

    xdr.ontimeout = function () {
        return console.error('hit timeout in XDR');
    };

    xdr.onerror = function () {
        return options.error('Server error, please wait a few minutes and try again');
    };

    xdr.onload = function () {
        return options.callback(JSON.parse(xdr.responseText));
    };

    xdr.onprogress = function () {
        return console.log('if you dont include this, the request will abort in IE9');
    };

    setTimeout(function () {
        return xdr.send(options.bodyParams);
    }, 100);
}

function makeStandardRequest(url, _ref2) {
    var callback = _ref2.callback,
        error = _ref2.error,
        method = _ref2.method,
        bodyParams = _ref2.bodyParams,
        headers = _ref2.headers;

    var defaults = {
        callback: function callback() {
            return null;
        },
        error: function error() {
            return null;
        },
        method: 'GET',
        bodyParams: null,
        headers: {}
    };

    var options = Object.assign({}, defaults, {
        callback: callback,
        error: error,
        method: method,
        bodyParams: bodyParams,
        headers: headers
    });

    var xmlhttp = new XMLHttpRequest();

    xmlhttp.addEventListener('load', function (evt) {
        if (xmlhttp.status >= 500) {
            options.error('Server error, please wait a few minutes and try again');
        } else if (xmlhttp.status >= 400) {
            options.error(xmlhttp.responseText);
        } else {
            options.callback(JSON.parse(xmlhttp.responseText));
        }
    });

    xmlhttp.open(options.method, url, true);

    if (headers) {
        Object.keys(headers).forEach(function (k) {
            return xmlhttp.setRequestHeader(k, options.headers[k]);
        });
    }

    xmlhttp.send(options.bodyParams);
}

function makeRequest() {
    var fn = window.XDomainRequest ? makeIERequest : makeStandardRequest;
    fn.apply(undefined, arguments);
}