const UglifyJSPlugin = require('uglifyjs-webpack-plugin');
const debug = process.env.NODE_ENV !== "production";
const webpack = require('webpack');

const productionPlugins = [
    new UglifyJSPlugin({
        output: {
            comments: false,
            beautify: false,
        },
    })
];

const debugPlugins = [];

// base plugins used in prod and dev
let plugins = [
    new webpack.EnvironmentPlugin({
        'BUILD_FOR_WINDOW': false
    }),
    new webpack.DefinePlugin({
        VERSION: JSON.stringify(require("./package.json").version)
    })
];

plugins = debug ? plugins.concat(debugPlugins) : plugins.concat(productionPlugins);


module.exports = {
    context: __dirname,

    devtool: debug ? "inline-sourcemap" : "source-map",

    entry: "./src/js/icarus.js",

    output: {
        path: __dirname + "/dist",
        filename: "icarus.min.js"
    },

    module: {
        rules: [
            {
                test: /\.js?$/,
                exclude: /(node_modules|bower_components)/,
                use: [
                    'babel-loader',
                    'eslint-loader'
                ]
            }
        ]
    },

    plugins: plugins
};